#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <errno.h>
#include <netinet/ip.h>

#include "server.h"

#define PIPE_NAME "/tmp/sw_pipe"


struct client client[MAX_CONNECT];


int my_listen( int port )
{
    int sd;                     // 待ち受けソケット
    struct sockaddr_in addr;    // ソケットアドレス
    int opt;
    int result;
    

    // listenポートの情報を設定
    memset(&addr,0,sizeof(addr));
    addr.sin_family      = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port        = htons(port);

    // 待ち受けソケット作成
    sd = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
    if( sd<0 ){
        perror("socket");
        return -1;
    }
    
    // ソケットオプション登録
    opt = 1;
    setsockopt( sd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt) );

    // バインド
    result = bind( sd, (struct sockaddr*)&addr, sizeof(addr) );
    if( result<0 ){
        perror("bind");
        close(sd);
        return -1;
    }

    // 待ち受け開始
    result = listen( sd, SOMAXCONN );
    if( result<0 ){
        perror("listen");
        close(sd);
        return -1;
    }
    
    return sd;
}

int my_accept( int listen_sd )
{
    int sd;                         // ソケット
    struct sockaddr_storage addr;   // ソケットアドレス
    socklen_t len;                  // ソケットアドレス長
    char host[256];                 // ホスト名表示用
    int result;
    
    
    // 待ち受け
    len = sizeof(addr);
    sd = accept( listen_sd, (struct sockaddr*)&addr, &len );
    if( sd<0 ){
        perror("accept");
        return -1;
    }
    
    // 相手の名前を表示（無くても構わない）
    result = getnameinfo( (struct sockaddr*)&addr, len, host, sizeof(host), NULL, 0, NI_NUMERICHOST );
    if( result!=0 ){
        fprintf( stderr,"getnameinfo: %s\n", gai_strerror(result) );
    }else{
        printf("connected by %s\n",host);
    }
    
    return sd;
}

//--------------------------------------------------------------



int main( int argc, char *argv[])
{
	int i;
    int listen_sd;  // 待ち受けソケット
	int pipe_fd;    // SWポーリングからの名前付きpipe

    signal(SIGPIPE, SIG_IGN);   // PIPE切断シグナルを無視する

	// SWポーリングのpipeを開く
	pipe_fd = open(PIPE_NAME, O_RDONLY);
	if(pipe_fd < 0){
		perror(PIPE_NAME);
		exit(1);
	}

	memset(&client, 0, sizeof(client));

    // 待ち受けソケット作成
    listen_sd = my_listen( MYPORT );
    if( listen_sd<0 ){
        exit(1);
    }


	/* メインループ */
	while(1){
		int result;
		fd_set fds;
		struct timeval tv;

		tv.tv_sec  = 10;
		tv.tv_usec = 0;

		FD_ZERO( &fds );
		FD_SET( listen_sd, &fds );
		FD_SET( pipe_fd, &fds );
		for(i=0; i<MAX_CONNECT; i++){
			if(client[i].sd){
				FD_SET(client[i].sd, &fds );	/* 接続済みソケットをFDSETする */
			}
		}

		printf("waiting\n");
		result = select( FD_SETSIZE, &fds, NULL, NULL, &tv );

		/* select()から抜けたところで場合分けをし、
		   イベントドリブンを作る */

		if( result<0 ){
			/* 負が返った時はエラー */
			if( errno==EINTR ){
				/* EINTR はシグナルキャッチ */
				continue;
			}
			perror("select");
			exit(1);
		}

		if( result == 0){
			/* 0はタイムアウト処理 */
			timeout_event();
			continue;
		}

		/* TCP新規接続処理 */
		if( FD_ISSET( listen_sd, &fds ) ){
			int sd = my_accept( listen_sd );
			for(i=0; i<MAX_CONNECT; i++){
				if(client[i].sd == 0){
					client[i].sd = sd;
					break;
				}
			}
			if( i==MAX_CONNECT ){
				printf("too many connect\n");
				close(sd);
			}
			continue;
	    }

		/* TCP接続済みソケットのチェック */
		for(i=0; i<MAX_CONNECT; i++){
			if(client[i].sd == 0)
				continue;
			if(FD_ISSET(client[i].sd, &fds ) ){
				result = data_event(&client[i]);
				if( result<0 ){
					close(client[i].sd);
					printf("close(%d)\n", client[i].sd);
					memset(&client[i], 0 , sizeof (struct client));
				}
			}
		}

		if( FD_ISSET( pipe_fd, &fds ) ){
			sw_event(pipe_fd);	/* SW押した/離した */
		}
    }
}



