#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#define SW0_DEVNAME "/dev/sw0"
#define SW1_DEVNAME "/dev/sw1"

#define PIPE_NAME "/tmp/sw_pipe"

const char *sw_devname[]={
	SW0_DEVNAME,
	SW1_DEVNAME
};

#define SIZE (sizeof(sw_devname)/sizeof(char*))

int old_val[SIZE];

int read_sw(int sw)
{
	int fd;
	int ret;
	char c;

	if(sw>=SIZE){
		return -EINVAL;
	}if(sw<0){
		return -EINVAL;
	}

	fd = open(sw_devname[sw],O_RDONLY);
	if(fd<0){
		perror(sw_devname[sw]);
		exit(1);
	}

	ret = read(fd, &c, 1);
	if(ret<0){
		perror(sw_devname[sw]);
		exit(2);
	}
	close(fd);


	if(c=='0')
		return 0;

	if(c=='1')
		return 1;

	return -EINVAL;

}

int switch_read(int fd_pipe, int sw)
{
	int val;
	int ret;
	int len;
	char str[16];

	val = read_sw(sw);

	if (old_val[sw] != val){
		char c;

		if(val==0)
			c='D';	// down
		else if(val==1)
			c='U';	// up
		else
			return 0;

		len = sprintf(str,"%c%c\n",sw+'0', c);
		printf("%s",str);
		ret = write(fd_pipe, str, len);
		if(ret<0){
			return -1;
		}
	}
	old_val[sw]=val;
	return 0;
}




int main(){
	int i;
	int fd_pipe;
	int ret;

	mkfifo(PIPE_NAME, 0644);

	while(1){
		printf("open(%s)\n",PIPE_NAME);
		fd_pipe = open(PIPE_NAME,O_WRONLY | O_NONBLOCK);
		if(fd_pipe<0){
			usleep(100*1000);	// 100msec
			continue;
		}
		while(1){
			for(i=0; i<SIZE; i++){
				ret = switch_read(fd_pipe, i);
				if(ret<0)
					break;
			}
			if(ret<0)
				break;
			usleep(50*1000);	// 50msec
		}
		close(fd_pipe);
	}

	return 0;
}

